==> atlassian-seraph-2.0-sources.jar: com/atlassian/seraph/config/SecurityConfigImpl.java <==
package com.atlassian.seraph.config;

import com.atlassian.seraph.Initable;
import com.atlassian.seraph.SecurityService;
import com.atlassian.seraph.auth.AuthenticationContext;
import com.atlassian.seraph.auth.AuthenticationContextImpl;
import com.atlassian.seraph.auth.Authenticator;
import com.atlassian.seraph.auth.RoleMapper;
import com.atlassian.seraph.controller.SecurityController;
import com.atlassian.seraph.cookie.CookieFactory;
import com.atlassian.seraph.interceptor.Interceptor;
import com.atlassian.seraph.util.XMLUtils;

import org.apache.log4j.Logger;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.opensymphony.util.ClassLoaderUtil;

import java.io.Serializable;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CopyOnWriteArrayList;

import javax.xml.parsers.DocumentBuilderFactory;

/**
 * The main class of Seraph's configuration.
 * <p>
 * This class is a Singleton, access it using SecurityConfigFactory.getInstance().
 */
public class SecurityConfigImpl implements Serializable, SecurityConfig
{
    private static final Logger log = Logger.getLogger(SecurityConfigImpl.class);

    public static final String DEFAULT_CONFIG_LOCATION = "seraph-config.xml";

    private static final int YEAR_IN_SECONDS = 365 * 24 * 60 * 60;

    private final Authenticator authenticator;
    private final RoleMapper roleMapper;
    private final SecurityController controller;
    private final List<SecurityService> services;
    private final List<Interceptor> interceptors = new CopyOnWriteArrayList<Interceptor>();

    private final String loginURL;
    private final String logoutURL;
    private final String originalURLKey;
    private final String cookieEncoding;
    private final String loginCookieKey;
    private final String linkLoginURL;
    private final String authType;
    private RedirectPolicy redirectPolicy;

    private boolean insecureCookie;
    // the age of the auto-login cookie - default = 1 year (in seconds)
    private final int autoLoginCookieAge;

    private final LoginUrlStrategy loginUrlStrategy;
    private final String loginCookiePath;

    public SecurityConfigImpl(String configFileLocation) throws ConfigurationException
    {
        if (configFileLocation != null)
        {
            if (SecurityConfigImpl.log.isDebugEnabled())
            {
                SecurityConfigImpl.log.debug("Config file location passed.  Location: " + configFileLocation);
            }
        }
        else
        {
            configFileLocation = "seraph-config.xml";
            if (SecurityConfigImpl.log.isDebugEnabled())
            {
                SecurityConfigImpl.log.debug("Initialising securityConfig using default configFile: " + configFileLocation);
            }
        }

        try
        {
            final DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            final URL fileUrl = ClassLoaderUtil.getResource(configFileLocation, getClass());

            if (fileUrl == null)
            {
                throw new IllegalArgumentException("No such XML file: " + configFileLocation);
            }

            // Parse document
            final Document doc = factory.newDocumentBuilder().parse(fileUrl.toString());
            final Element rootEl = doc.getDocumentElement();

            final NodeList nl = rootEl.getElementsByTagName("parameters");
            final Element parametersEl = (Element) nl.item(0);
            final Map<String, String> globalParams = getInitParameters(parametersEl);

            loginURL = globalParams.get("login.url");
            linkLoginURL = globalParams.get("link.login.url");
            logoutURL = globalParams.get("logout.url");
            cookieEncoding = globalParams.get("cookie.encoding");
            loginCookiePath = globalParams.get("login.cookie.path");
            authType = globalParams.get("authentication.type");
            insecureCookie = "true".equals(globalParams.get("insecure.cookie"));

            if (globalParams.get("original.url.key") != null)
            {
                originalURLKey = globalParams.get("original.url.key");
            }
            else
            {
                originalURLKey = "seraph_originalurl";
            }

            if (globalParams.get("login.cookie.key") != null)
            {
                loginCookieKey = globalParams.get("login.cookie.key");
            }
            else
            {
                loginCookieKey = "seraph.os.cookie";
            }

            if (globalParams.get("autologin.cookie.age") != null)
            {
                autoLoginCookieAge = Integer.parseInt(globalParams.get("autologin.cookie.age").toString());
            }
            else
            {
                autoLoginCookieAge = SecurityConfigImpl.YEAR_IN_SECONDS;
            }

            // be VERY careful about changing order here, THIS reference is passed out while initializing and so we are partially constructed when
            // clients call us

            authenticator = configureAuthenticator(rootEl);
            controller = configureController(rootEl);
            roleMapper = configureRoleMapper(rootEl);
            services = Collections.unmodifiableList(configureServices(rootEl));
            configureInterceptors(rootEl);
            loginUrlStrategy = configureLoginUrlStrategy(rootEl);
            configureRedirectPolicy(rootEl);
            CookieFactory.init(this);
        }
        catch (final Exception e)
        {
            throw new ConfigurationException("Exception configuring from '" + configFileLocation, e);
        }
    }

    protected void configureRedirectPolicy(final Element rootEl) throws ConfigurationException
    {
        // If no redirect-policy is configured, then this will be null which is valid.
        redirectPolicy = (RedirectPolicy) configureClass(rootEl, "redirect-policy", this);

        // If none is configured, then use the default.
        if (redirectPolicy == null)
        {
            redirectPolicy = new DefaultRedirectPolicy();
        }
    }

    private LoginUrlStrategy configureLoginUrlStrategy(final Element rootEl) throws ConfigurationException
    {
        LoginUrlStrategy loginUrlStrategy = (LoginUrlStrategy) configureClass(rootEl, "login-url-strategy", this);

        if (loginUrlStrategy == null)
        {
            loginUrlStrategy = new DefaultLoginUrlStrategy();
        }
        return loginUrlStrategy;
    }

    private Authenticator configureAuthenticator(final Element rootEl) throws ConfigurationException
    {
        Authenticator authenticator = (Authenticator) configureClass(rootEl, "authenticator", this);

        try
        {
            if (authenticator == null)
            {
                authenticator = (Authenticator) ClassLoaderUtil.loadClass(Authenticator.DEFAULT_AUTHENTICATOR, getClass()).newInstance();
                authenticator.init(Collections.<String, String> emptyMap(), this);
            }
        }
        catch (final Exception e)
        {
            throw new ConfigurationException("Could not lookup class: " + Authenticator.DEFAULT_AUTHENTICATOR, e);
        }
        return authenticator;
    }

    private SecurityController configureController(final Element rootEl) throws ConfigurationException
    {
        SecurityController controller = (SecurityController) configureClass(rootEl, "controller", this);

        try
        {
            if (controller == null)
            {
                controller = (SecurityController) ClassLoaderUtil.loadClass(SecurityController.NULL_CONTROLLER, getClass()).newInstance();
            }
        }
        catch (final Exception e)
        {
            throw new ConfigurationException("Could not lookup class: " + SecurityController.NULL_CONTROLLER, e);
        }
        return controller;
    }

    private RoleMapper configureRoleMapper(final Element rootEl) throws ConfigurationException
    {
        return (RoleMapper) configureClass(rootEl, "rolemapper", this);
    }

    private static Initable configureClass(final Element rootEl, final String tagname, final SecurityConfig owner) throws ConfigurationException
    {
        try
        {
            final NodeList elementList = rootEl.getElementsByTagName(tagname);

            for (int i = 0; i < elementList.getLength(); i++)
            {
                final Element authEl = (Element) elementList.item(i);
                final String clazz = authEl.getAttribute("class");

                final Initable initable = (Initable) ClassLoaderUtil.loadClass(clazz, owner.getClass()).newInstance();

                initable.init(getInitParameters(authEl), owner);
                return initable;
            }
            return null;
        }
        catch (final Exception e)
        {
            throw new ConfigurationException("Could not create: " + tagname, e);
        }
    }

    // only called from the constructor
    private List<SecurityService> configureServices(final Element rootEl) throws ConfigurationException
    {
        final NodeList nl = rootEl.getElementsByTagName("services");
        final List<SecurityService> result = new ArrayList<SecurityService>();

        if ((nl != null) && (nl.getLength() > 0))
        {
            final Element servicesEl = (Element) nl.item(0);
            final NodeList serviceList = servicesEl.getElementsByTagName("service");

            for (int i = 0; i < serviceList.getLength(); i++)
            {
                final Element serviceEl = (Element) serviceList.item(i);
                final String serviceClazz = serviceEl.getAttribute("class");

                if ((serviceClazz == null) || "".equals(serviceClazz))
                {
                    throw new ConfigurationException("Service element with bad class attribute");
                }

                try
                {
                    SecurityConfigImpl.log.debug("Adding seraph service of class: " + serviceClazz);
                    final SecurityService service = (SecurityService) ClassLoaderUtil.loadClass(serviceClazz, getClass()).newInstance();

                    service.init(getInitParameters(serviceEl), this);

                    result.add(service);
                }
                catch (final Exception e)
                {
                    throw new ConfigurationException("Could not getRequest service: " + serviceClazz, e);
                }
            }
        }
        return result;
    }

    private void configureInterceptors(final Element rootEl) throws ConfigurationException
    {
        final NodeList nl = rootEl.getElementsByTagName("interceptors");

        if ((nl != null) && (nl.getLength() > 0))
        {
            final Element interceptorsEl = (Element) nl.item(0);
            final NodeList interceptorList = interceptorsEl.getElementsByTagName("interceptor");

            for (int i = 0; i < interceptorList.getLength(); i++)
            {
                final Element interceptorEl = (Element) interceptorList.item(i);
                final String interceptorClazz = interceptorEl.getAttribute("class");

                if ((interceptorClazz == null) || "".equals(interceptorClazz))
                {
                    throw new ConfigurationException("Interceptor element with bad class attribute");
                }

                try
                {
                    SecurityConfigImpl.log.debug("Adding interceptor of class: " + interceptorClazz);
                    final Interceptor interceptor = (Interceptor) ClassLoaderUtil.loadClass(interceptorClazz, getClass()).newInstance();

                    interceptor.init(getInitParameters(interceptorEl), this);

                    interceptors.add(interceptor);
                }
                catch (final Exception e)
                {
                    throw new ConfigurationException("Could not getRequest service: " + interceptorClazz, e);
                }
            }
        }
    }

    /**
     * Returns a Map of the "init-param" properties under the given element.
     * The map could be empty, but is guaranteed not to be null.
     * 
     * @param el The XML config element.
     * @return a Map of the "init-param" properties under the given element.
     */
    private static Map<String, String> getInitParameters(final Element el)
    {
        final Map<String, String> params = new HashMap<String, String>();
        final NodeList nl = el.getElementsByTagName("init-param");

        for (int i = 0; i < nl.getLength(); i++)
        {
            final Node initParam = nl.item(i);
            final String paramName = XMLUtils.getContainedText(initParam, "param-name");
            final String paramValue = XMLUtils.getContainedText(initParam, "param-value");
            params.put(paramName, paramValue);
        }
        return Collections.unmodifiableMap(params);
    }

    public void destroy()
    {
        for (final Object element : services)
        {
            final SecurityService securityService = (SecurityService) element;
            securityService.destroy();
        }

        for (final Object element : interceptors)
        {
            ((Interceptor) element).destroy();
        }
    }

    /**
     * Do not use in production! Only used in tests, will be removed.
     */
    public void addInterceptor(final Interceptor interceptor)
    {
        interceptors.add(interceptor);
    }

    public List<SecurityService> getServices()
    {
        return services;
    }

    public String getLoginURL()
    {
        return loginUrlStrategy.getLoginURL(this, loginURL);
    }

    public String getLinkLoginURL()
    {
        return loginUrlStrategy.getLinkLoginURL(this, linkLoginURL);
    }

    public String getLogoutURL()
    {
        return loginUrlStrategy.getLogoutURL(this, logoutURL);
    }

    public String getOriginalURLKey()
    {
        return originalURLKey;
    }

    public Authenticator getAuthenticator()
    {
        return authenticator;
    }

    public AuthenticationContext getAuthenticationContext()
    {
        return new AuthenticationContextImpl(); // todo - load this from a config file
    }

    public SecurityController getController()
    {
        return controller;
    }

    public RoleMapper getRoleMapper()
    {
        return roleMapper;
    }

    public RedirectPolicy getRedirectPolicy()
    {
        return redirectPolicy;
    }

    public <T extends Interceptor> List<T> getInterceptors(final Class<T> desiredInterceptorClass)
    {
        final List<T> result = new ArrayList<T>();
        for (final Interceptor interceptor : interceptors)
        {
            if (desiredInterceptorClass.isAssignableFrom(interceptor.getClass()))
            {
                result.add(desiredInterceptorClass.cast(interceptor));
            }
        }
        return Collections.unmodifiableList(result);
    }

    public String getCookieEncoding()
    {
        return cookieEncoding;
    }

    public String getLoginCookiePath()
    {
        return loginCookiePath;
    }

    public String getLoginCookieKey()
    {
        return loginCookieKey;
    }

    public String getAuthType()
    {
        return authType;
    }

    public boolean isInsecureCookie()
    {
        return insecureCookie;
    }

    public int getAutoLoginCookieAge()
    {
        return autoLoginCookieAge;
    }
}

==> atlassian-seraph-2.1.3-sources.jar: com/atlassian/seraph/config/SecurityConfigImpl.java <==
package com.atlassian.seraph.config;

import com.atlassian.seraph.Initable;
import com.atlassian.seraph.SecurityService;
import com.atlassian.seraph.auth.AuthenticationContext;
import com.atlassian.seraph.auth.AuthenticationContextImpl;
import com.atlassian.seraph.auth.Authenticator;
import com.atlassian.seraph.auth.RoleMapper;
import com.atlassian.seraph.controller.SecurityController;
import com.atlassian.seraph.cookie.CookieFactory;
import com.atlassian.seraph.elevatedsecurity.ElevatedSecurityGuard;
import com.atlassian.seraph.elevatedsecurity.NoopElevatedSecurityGuard;
import com.atlassian.seraph.interceptor.Interceptor;
import com.atlassian.seraph.util.XMLUtils;
import com.opensymphony.util.ClassLoaderUtil;
import org.apache.log4j.Logger;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import java.io.Serializable;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CopyOnWriteArrayList;
import javax.xml.parsers.DocumentBuilderFactory;

/**
 * The main class of Seraph's configuration.
 * <p>
 * This class is a Singleton, access it using SecurityConfigFactory.getInstance().
 */
public class SecurityConfigImpl implements Serializable, SecurityConfig
{
    private static final Logger log = Logger.getLogger(SecurityConfigImpl.class);

    public static final String DEFAULT_CONFIG_LOCATION = "seraph-config.xml";

    private static final int YEAR_IN_SECONDS = 365 * 24 * 60 * 60;

    private final Authenticator authenticator;
    private final ElevatedSecurityGuard elevatedSecurityGuard;
    private final RoleMapper roleMapper;
    private final SecurityController controller;
    private final List<SecurityService> services;
    private final List<Interceptor> interceptors = new CopyOnWriteArrayList<Interceptor>();

    private final String loginURL;
    private final String logoutURL;
    private final String originalURLKey;
    private final String cookieEncoding;
    private final String loginCookieKey;
    private final String linkLoginURL;
    private final String authType;
    private RedirectPolicy redirectPolicy;

    private boolean insecureCookie;
    // the age of the auto-login cookie - default = 1 year (in seconds)
    private final int autoLoginCookieAge;

    private final LoginUrlStrategy loginUrlStrategy;
    private final String loginCookiePath;

    public SecurityConfigImpl(String configFileLocation) throws ConfigurationException
    {
        if (configFileLocation != null)
        {
            if (SecurityConfigImpl.log.isDebugEnabled())
            {
                SecurityConfigImpl.log.debug("Config file location passed.  Location: " + configFileLocation);
            }
        }
        else
        {
            configFileLocation = "seraph-config.xml";
            if (SecurityConfigImpl.log.isDebugEnabled())
            {
                SecurityConfigImpl.log.debug("Initialising securityConfig using default configFile: " + configFileLocation);
            }
        }

        try
        {
            final DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
            final URL fileUrl = ClassLoaderUtil.getResource(configFileLocation, getClass());

            if (fileUrl == null)
            {
                throw new IllegalArgumentException("No such XML file: " + configFileLocation);
            }

            // Parse document
            final Document doc = factory.newDocumentBuilder().parse(fileUrl.toString());
            final Element rootEl = doc.getDocumentElement();

            final NodeList nl = rootEl.getElementsByTagName("parameters");
            final Element parametersEl = (Element) nl.item(0);
            final Map<String, String> globalParams = getInitParameters(parametersEl);

            loginURL = globalParams.get("login.url");
            linkLoginURL = globalParams.get("link.login.url");
            logoutURL = globalParams.get("logout.url");
            cookieEncoding = globalParams.get("cookie.encoding");
            loginCookiePath = globalParams.get("login.cookie.path");
            authType = globalParams.get("authentication.type");
            insecureCookie = "true".equals(globalParams.get("insecure.cookie"));

            if (globalParams.get("original.url.key") != null)
            {
                originalURLKey = globalParams.get("original.url.key");
            }
            else
            {
                originalURLKey = "seraph_originalurl";
            }

            if (globalParams.get("login.cookie.key") != null)
            {
                loginCookieKey = globalParams.get("login.cookie.key");
            }
            else
            {
                loginCookieKey = "seraph.os.cookie";
            }

            if (globalParams.get("autologin.cookie.age") != null)
            {
                autoLoginCookieAge = Integer.parseInt(globalParams.get("autologin.cookie.age"));
            }
            else
            {
                autoLoginCookieAge = SecurityConfigImpl.YEAR_IN_SECONDS;
            }

            // be VERY careful about changing order here, THIS reference is passed out while initializing and so we are partially constructed when
            // clients call us

            authenticator = configureAuthenticator(rootEl);
            controller = configureController(rootEl);
            roleMapper = configureRoleMapper(rootEl);
            services = Collections.unmodifiableList(configureServices(rootEl));
            configureInterceptors(rootEl);
            loginUrlStrategy = configureLoginUrlStrategy(rootEl);
            configureRedirectPolicy(rootEl);
            CookieFactory.init(this);
            elevatedSecurityGuard = configureElevatedSecurityGuard(rootEl);
        }
        catch (final Exception e)
        {
            throw new ConfigurationException("Exception configuring from '" + configFileLocation, e);
        }
    }

    protected void configureRedirectPolicy(final Element rootEl) throws ConfigurationException
    {
        // If no redirect-policy is configured, then this will be null which is valid.
        redirectPolicy = (RedirectPolicy) configureClass(rootEl, "redirect-policy", this);

        // If none is configured, then use the default.
        if (redirectPolicy == null)
        {
            redirectPolicy = new DefaultRedirectPolicy();
        }
    }

    private LoginUrlStrategy configureLoginUrlStrategy(final Element rootEl) throws ConfigurationException
    {
        LoginUrlStrategy loginUrlStrategy = (LoginUrlStrategy) configureClass(rootEl, "login-url-strategy", this);

        if (loginUrlStrategy == null)
        {
            loginUrlStrategy = new DefaultLoginUrlStrategy();
        }
        return loginUrlStrategy;
    }

    private Authenticator configureAuthenticator(final Element rootEl) throws ConfigurationException
    {
        Authenticator authenticator = (Authenticator) configureClass(rootEl, "authenticator", this);

        try
        {
            if (authenticator == null)
            {
                authenticator = (Authenticator) ClassLoaderUtil.loadClass(Authenticator.DEFAULT_AUTHENTICATOR, getClass()).newInstance();
                authenticator.init(Collections.<String, String> emptyMap(), this);
            }
        }
        catch (final Exception e)
        {
            throw new ConfigurationException("Could not lookup class: " + Authenticator.DEFAULT_AUTHENTICATOR, e);
        }
        return authenticator;
    }

    private ElevatedSecurityGuard configureElevatedSecurityGuard(final Element rootEl) throws ConfigurationException
    {
        ElevatedSecurityGuard elevatedSecurityGuard = (ElevatedSecurityGuard) configureClass(rootEl, "elevatedsecurityguard", this);
        if (elevatedSecurityGuard == null)
        {
            elevatedSecurityGuard = NoopElevatedSecurityGuard.INSTANCE;
        }
        return elevatedSecurityGuard;
    }

    private SecurityController configureController(final Element rootEl) throws ConfigurationException
    {
        SecurityController controller = (SecurityController) configureClass(rootEl, "controller", this);

        try
        {
            if (controller == null)
            {
                controller = (SecurityController) ClassLoaderUtil.loadClass(SecurityController.NULL_CONTROLLER, getClass()).newInstance();
            }
        }
        catch (final Exception e)
        {
            throw new ConfigurationException("Could not lookup class: " + SecurityController.NULL_CONTROLLER, e);
        }
        return controller;
    }

    private RoleMapper configureRoleMapper(final Element rootEl) throws ConfigurationException
    {
        return (RoleMapper) configureClass(rootEl, "rolemapper", this);
    }

    private static Initable configureClass(final Element rootEl, final String tagname, final SecurityConfig owner) throws ConfigurationException
    {
        try
        {
            final NodeList elementList = rootEl.getElementsByTagName(tagname);

            for (int i = 0; i < elementList.getLength(); i++)
            {
                final Element authEl = (Element) elementList.item(i);
                final String clazz = authEl.getAttribute("class");

                final Initable initable = (Initable) ClassLoaderUtil.loadClass(clazz, owner.getClass()).newInstance();

                initable.init(getInitParameters(authEl), owner);
                return initable;
            }
            return null;
        }
        catch (final Exception e)
        {
            throw new ConfigurationException("Could not create: " + tagname, e);
        }
    }

    // only called from the constructor
    private List<SecurityService> configureServices(final Element rootEl) throws ConfigurationException
    {
        final NodeList nl = rootEl.getElementsByTagName("services");
        final List<SecurityService> result = new ArrayList<SecurityService>();

        if ((nl != null) && (nl.getLength() > 0))
        {
            final Element servicesEl = (Element) nl.item(0);
            final NodeList serviceList = servicesEl.getElementsByTagName("service");

            for (int i = 0; i < serviceList.getLength(); i++)
            {
                final Element serviceEl = (Element) serviceList.item(i);
                final String serviceClazz = serviceEl.getAttribute("class");

                if ((serviceClazz == null) || "".equals(serviceClazz))
                {
                    throw new ConfigurationException("Service element with bad class attribute");
                }

                try
                {
                    SecurityConfigImpl.log.debug("Adding seraph service of class: " + serviceClazz);
                    final SecurityService service = (SecurityService) ClassLoaderUtil.loadClass(serviceClazz, getClass()).newInstance();

                    service.init(getInitParameters(serviceEl), this);

                    result.add(service);
                }
                catch (final Exception e)
                {
                    throw new ConfigurationException("Could not getRequest service: " + serviceClazz, e);
                }
            }
        }
        return result;
    }

    private void configureInterceptors(final Element rootEl) throws ConfigurationException
    {
        final NodeList nl = rootEl.getElementsByTagName("interceptors");

        if ((nl != null) && (nl.getLength() > 0))
        {
            final Element interceptorsEl = (Element) nl.item(0);
            final NodeList interceptorList = interceptorsEl.getElementsByTagName("interceptor");

            for (int i = 0; i < interceptorList.getLength(); i++)
            {
                final Element interceptorEl = (Element) interceptorList.item(i);
                final String interceptorClazz = interceptorEl.getAttribute("class");

                if ((interceptorClazz == null) || "".equals(interceptorClazz))
                {
                    throw new ConfigurationException("Interceptor element with bad class attribute");
                }

                try
                {
                    SecurityConfigImpl.log.debug("Adding interceptor of class: " + interceptorClazz);
                    final Interceptor interceptor = (Interceptor) ClassLoaderUtil.loadClass(interceptorClazz, getClass()).newInstance();

                    interceptor.init(getInitParameters(interceptorEl), this);

                    interceptors.add(interceptor);
                }
                catch (final Exception e)
                {
                    throw new ConfigurationException("Could not getRequest service: " + interceptorClazz, e);
                }
            }
        }
    }

    /**
     * Returns a Map of the "init-param" properties under the given element.
     * The map could be empty, but is guaranteed not to be null.
     * 
     * @param el The XML config element.
     * @return a Map of the "init-param" properties under the given element.
     */
    private static Map<String, String> getInitParameters(final Element el)
    {
        final Map<String, String> params = new HashMap<String, String>();
        final NodeList nl = el.getElementsByTagName("init-param");

        for (int i = 0; i < nl.getLength(); i++)
        {
            final Node initParam = nl.item(i);
            final String paramName = XMLUtils.getContainedText(initParam, "param-name");
            final String paramValue = XMLUtils.getContainedText(initParam, "param-value");
            params.put(paramName, paramValue);
        }
        return Collections.unmodifiableMap(params);
    }

    public void destroy()
    {
        for (final Object element : services)
        {
            final SecurityService securityService = (SecurityService) element;
            securityService.destroy();
        }

        for (final Object element : interceptors)
        {
            ((Interceptor) element).destroy();
        }
    }

    /**
     * Do not use in production! Only used in tests, will be removed.
     * @param interceptor the Interceptor to add
     */
    public void addInterceptor(final Interceptor interceptor)
    {
        interceptors.add(interceptor);
    }

    public List<SecurityService> getServices()
    {
        return services;
    }

    public String getLoginURL()
    {
        return loginUrlStrategy.getLoginURL(this, loginURL);
    }

    public String getLinkLoginURL()
    {
        return loginUrlStrategy.getLinkLoginURL(this, linkLoginURL);
    }

    public String getLogoutURL()
    {
        return loginUrlStrategy.getLogoutURL(this, logoutURL);
    }

    public String getOriginalURLKey()
    {
        return originalURLKey;
    }

    public Authenticator getAuthenticator()
    {
        return authenticator;
    }

    public AuthenticationContext getAuthenticationContext()
    {
        return new AuthenticationContextImpl(); // todo - load this from a config file
    }

    public SecurityController getController()
    {
        return controller;
    }

    public RoleMapper getRoleMapper()
    {
        return roleMapper;
    }

    public RedirectPolicy getRedirectPolicy()
    {
        return redirectPolicy;
    }

    public <T extends Interceptor> List<T> getInterceptors(final Class<T> desiredInterceptorClass)
    {
        final List<T> result = new ArrayList<T>();
        for (final Interceptor interceptor : interceptors)
        {
            if (desiredInterceptorClass.isAssignableFrom(interceptor.getClass()))
            {
                result.add(desiredInterceptorClass.cast(interceptor));
            }
        }
        return Collections.unmodifiableList(result);
    }

    public String getCookieEncoding()
    {
        return cookieEncoding;
    }

    public String getLoginCookiePath()
    {
        return loginCookiePath;
    }

    public String getLoginCookieKey()
    {
        return loginCookieKey;
    }

    public String getAuthType()
    {
        return authType;
    }

    public boolean isInsecureCookie()
    {
        return insecureCookie;
    }

    public int getAutoLoginCookieAge()
    {
        return autoLoginCookieAge;
    }

    public ElevatedSecurityGuard getElevatedSecurityGuard()
    {
        return elevatedSecurityGuard;
    }
}
